/* safestr.h.  Generated by configure.  */
#ifndef SAFESTR_H
#define SAFESTR_H

#ifndef WIN32
/* BEGIN:   autoconf substitutions */
#define HAVE_SYS_TYPES_H 1
#define HAVE_U_INT32_T 1
#define STDC_HEADERS 1
#define HAVE_VSNPRINTF 1
/* #undef size_t */
/* #undef volatile */
/* END:     autoconf substitutions */

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef STDC_HEADERS
#include <stdio.h>
#include <stdarg.h>
#include <stddef.h>
#endif
#else
#include <stdio.h>
#include <stdarg.h>
#endif

/* This needs to be changed on platforms without a 32-bit int size.
 * It also needs to be commented out on platforms that don't have it.
 */
#if defined(WIN32) || !defined(HAVE_U_INT32_T)
typedef unsigned int u_int32_t;
#endif

#define SAFESTR_ERROR_NONE                  0
#define SAFESTR_ERROR_OUT_OF_MEMORY         0x80000001
#define SAFESTR_ERROR_INVALID_PARAMETER     0x80000002
#define SAFESTR_ERROR_BAD_ADDRESS           0x80000003
#define SAFESTR_ERROR_IMMUTABLE_STRING      0x80000004
#define SAFESTR_ERROR_INDEX_OUT_OF_RANGE    0x80000005
#define SAFESTR_ERROR_PRNG_FAILURE          0x80000006
#define SAFESTR_ERROR_INVALID_FORMAT_STRING 0x80000007
#define SAFESTR_ERROR_TOO_MANY_FORMAT_ARGS  0x80000008
#define SAFESTR_ERROR_ILLEGAL_PERCENT_N     0x80000009
#define SAFESTR_ERROR_INVALID_FORMAT_ARG    0x8000000A
#define SAFESTR_ERROR_STRING_TOO_LONG       0x8000000B
#define SAFESTR_ERROR_NOTFOUND              0xFFFFFFFF

#define SAFESTR_COMPARE_NOCASE    0x1 /* do NOT consider case in compares */
#define SAFESTR_COMPARE_LIMIT     0x2 /* restrict comparison to length */

#define SAFESTR_TEMPORARY         0x1
#define SAFESTR_IMMUTABLE         0x2
#define SAFESTR_TRUSTED           0x4

#define SAFESTR_COPY_LIMIT        0x1

#define SAFESTR_FIND_REVERSE      0x01 /* start search from end of string */
#define SAFESTR_FIND_FROMCHAR     0x02 /* start search from specified position */
#define SAFESTR_FIND_FROMNTH      0x04 /* start search from nth occurrence */
#define SAFESTR_FIND_NTH          0x08 /* find the nth occurrence */
#define SAFESTR_FIND_NOMATCHCASE  0x10 /* search case insensitively */

/* Convenience Macros */
#define SAFESTR_TEMP(x)           safestr_create((x), SAFESTR_TEMPORARY | \
                                                      SAFESTR_IMMUTABLE)
#define SAFESTR_TEMP_TRUSTED(x)   safestr_create((x), SAFESTR_TEMPORARY | \
                                                      SAFESTR_IMMUTABLE | \
                                                      SAFESTR_TRUSTED)
#define safestr_release(x)        safestr_free((x))

#define safestr_append(x, y)      safestr_concatenate((x), (y), 0)
#define safestr_nappend(x, y, z)  safestr_concatenate((x), (y), SAFESTR_COPY_LIMIT, (z))

#define safestr_find(x, y)        safestr_search((x), (y), 0)
#define safestr_rfind(x, y)       safestr_search((x), (y), SAFESTR_FIND_REVERSE)

#define safestr_copy(x, y)        safestr_duplicate((x), (y), 0)
#define safestr_ncopy(x, y, z)    safestr_duplicate((x), (y), SAFESTR_COPY_LIMIT, (z))

#define safestr_ncompare(x, y, z) safestr_compare((x), (y), SAFESTR_COMPARE_LIMIT, (z))

#ifdef __cplusplus
extern "C" {
#endif

typedef struct safestr_t
{
    char s;
} *safestr_t;

typedef void *  (*safestr_malloc_t) (size_t);
typedef void *  (*safestr_realloc_t)(void *, size_t);
typedef void    (*safestr_free_t)   (void *);

extern safestr_t safestr_alloc          (u_int32_t, u_int32_t);
extern char      safestr_charat         (safestr_t, u_int32_t);
extern safestr_t safestr_clone          (safestr_t, u_int32_t);
extern int       safestr_compare        (safestr_t, safestr_t, u_int32_t, ...);
extern void      safestr_concatenate    (safestr_t *, safestr_t, u_int32_t, ...);
extern safestr_t *safestr_convertarray  (char **, u_int32_t);
extern safestr_t safestr_create         (char *, u_int32_t);
extern void      safestr_delete         (safestr_t *, u_int32_t, u_int32_t);
extern void      safestr_duplicate      (safestr_t *, safestr_t, u_int32_t, ...);
extern int       safestr_equal          (safestr_t, safestr_t, u_int32_t, ...);
extern int       safestr_fprintf        (FILE *, safestr_t, ...);
extern void      safestr_free           (safestr_t);
extern void      safestr_insert         (safestr_t *, u_int32_t, safestr_t);
extern safestr_t safestr_join           (safestr_t *, safestr_t);
extern u_int32_t safestr_length         (safestr_t);
extern void      safestr_memzero        (volatile void *, u_int32_t);
extern int       safestr_printf         (safestr_t, ...);
extern u_int32_t safestr_reference      (safestr_t);
extern safestr_t safestr_replace        (safestr_t, safestr_t, safestr_t);
extern u_int32_t safestr_search         (safestr_t, safestr_t, u_int32_t, ...);
extern void      safestr_setcharat      (safestr_t, u_int32_t, char, int);
extern void      safestr_setmemfns      (safestr_malloc_t, safestr_realloc_t, safestr_free_t);
extern safestr_t safestr_slice          (safestr_t, u_int32_t, u_int32_t);
extern safestr_t *safestr_split         (safestr_t, safestr_t);
#ifdef HAVE_VSNPRINTF
extern u_int32_t safestr_sprintf        (safestr_t *, safestr_t, ...);
#endif
extern char      *safestr_strdup        (char *);
extern void      safestr_truncate       (safestr_t *, u_int32_t);
extern int       safestr_vfprintf       (FILE *, safestr_t fmt, va_list);
extern int       safestr_vprintf        (safestr_t, va_list);
#ifdef HAVE_VSNPRINTF
extern u_int32_t safestr_vsprintf       (safestr_t *, safestr_t, va_list);
#endif

extern void      safestr_trust          (safestr_t);
extern void      safestr_untrust        (safestr_t);
extern int       safestr_istrusted      (safestr_t);
extern void      safestr_makereadonly   (safestr_t);
extern void      safestr_makewritable   (safestr_t);
extern int       safestr_isreadonly     (safestr_t);

#ifdef __cplusplus
}
#endif

#endif /* SAFESTR_H */
